//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2002                 www.xilinx.com
//
//  XAPP 224 - Data Recovery
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       tb_top_v2_fast.v
//
//  Description :     Test bench for top_v2_fast.v
//
//  Date - revision : May 16th 2003 - v 2.2
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors make and you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specifically disclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does not warrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designs will be
//              uninterrupted or error free, or that defects in the Designs
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results of the
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or for any
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on any theory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure of the
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module tb_top_v2_fast ;

reg dataclk, clk, fastclk, match ;
wire datainb;
reg  reset ;
reg [63:0] datain ;
wire [1:0] dummy, dvalid ;
wire locked ;
reg [63:0] rxdata  ;
reg [7:0] count, counth ;

assign datainb = datain[63] ; 
	   
always @(posedge dataclk or posedge reset)
begin
if (reset)
	datain <= 64'b1111111011011100101110101001100001110110010101000011001000010000 ;
else
	datain <= {datain[62:0],datain[63]} ;
end

always @ (posedge clk or posedge locked or posedge reset)
begin
if (~locked | reset) begin
	rxdata <= 64'b0000000000000000000000000000000000000000000000000000000000000000  ;
	count <= 8'b00000000 ;
	counth <= 8'b00000000 ;
end else begin
	if (dvalid == 2'b01) begin 
		count <= count + 1'b1 ;
		rxdata <= {rxdata[62:0],dummy[0]} ;
	end else if (dvalid == 2'b11) begin
		count <= count + 2'b10 ;
		rxdata <= {rxdata[61:0], dummy[0],dummy[1]} ;
	end 
	if (rxdata == 64'b1111111011011100101110101001100001110110010101000011001000010000) begin
		match <= 1'b1 ;
		counth <= count ;
		count <= 8'b00000000 ;		
	end else begin
		match <= 1'b0 ;
	end
end 
end
initial clk = 0 ;
initial dataclk = 0 ;

always #(2500) dataclk = ~dataclk ; // 200 MHz clock for data transmission
// always #(2503) clk = ~clk ;		// slower clock for data reception
always #(2497) clk = ~clk  ;		// faster clock for data reception


top_v2_fast chip (
	.clkin(clk),
	.rstin(reset),
	.datain(datainb),
	.dummy(dummy),
	.dvalid(dvalid),
	.locked(locked)); 


initial
begin
reset = 1'b1 ;

#50000
reset = 1'b0 ;

//$stop ;
end 

endmodule