------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 224 - Data Recovery
--
------------------------------------------------------------------------------
--
--  File name :       top_v2_fast.v
--
--  Description :     Generic top level file to use data recovery module in a Virtex2,
-- 		      uses two dcms to generate two 90 degree phased clocks at up to 210 Mhz in a -5 .
--
--  Date - revision : May 16th 2003 - v 2.2
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_v2_fast is port (
	clkin 	: in std_logic;				-- clock input
	rstin 	: in std_logic;				-- reset input
	datain 	: in std_logic;				-- data input
	dummy 	: out std_logic_vector(1 downto 0);	-- dummy output for recovered data
	dvalid 	: out std_logic_vector(1 downto 0);	-- recovered data valid when high
	locked	: out std_logic); 			-- DCMs locked
end top_v2_fast;

architecture arch_top_v2_fast of top_v2_fast is
-- DCM primitive
component dcm 
--pragma translate_off
	generic (
	DLL_FREQUENCY_MODE	: string ;
	CLKOUT_PHASE_SHIFT 	: string ;
	PHASE_SHIFT	 	: integer );
--pragma translate_on
	port (
	clkin   : in  std_logic ;
	CLKFB   : in  std_logic ;
	DSSEN 	: in  std_logic ;
	PSINCDEC: in  std_logic ;
	PSEN 	: in  std_logic ;
	PSCLK 	: in  std_logic ;
	RST     : in  std_logic ;
	CLK0    : out std_logic ;
	CLK90   : out std_logic ;
	CLK180  : out std_logic ;
	CLK270  : out std_logic ;
	CLK2X   : out std_logic ;
	CLK2X180: out std_logic ;
	CLKDV   : out std_logic ;
	CLKFX   : out std_logic ;
	CLKFX180: out std_logic ;
	LOCKED  : out std_logic ;
	PSDONE  : out std_logic ;
	STATUS  : out std_logic_vector(7 downto 0)); 
end component;
-- BUFG primitive
component bufg port (i : in std_logic; o : out std_logic); end component;
-- IBUFG primitive
component ibufg port (i : in std_logic; o : out std_logic); end component;
-- Data recovery module
component data_recovery_virtex2_fast port (
	clk 	: in std_logic;
	clk90 	: in std_logic;
	data 	: in std_logic;
	rst 	: in std_logic;
	sdata 	: out std_logic_vector(1 downto 0) ;
	dv 	: out std_logic_vector(1 downto 0)) ; 
end component;

signal clk   	: std_logic;
signal low   	: std_logic;
signal clkdcm	: std_logic;
signal clkint	: std_logic;
signal clk90	: std_logic;
signal clk90dcm : std_logic;
signal dummyint : std_logic_vector(1 downto 0) ;
signal dvalidint: std_logic_vector(1 downto 0) ;
signal locked0  : std_logic ;
signal locked1  : std_logic ;
constant cps  	: string := "FIXED" ;
constant cph  	: string := "HIGH" ;
constant ps0  	: integer := 0 ;
constant ps1  	: integer := 64 ;

attribute DLL_FREQUENCY_MODE 	: string;
attribute CLKOUT_PHASE_SHIFT 	: string;
attribute PHASE_SHIFT 		: string;
attribute CLKIN_PERIOD 		: string ;

attribute CLKIN_PERIOD 		of dcm_0	: label is "2.5" ;
attribute DLL_FREQUENCY_MODE 	of dcm_0	: label is "HIGH";

attribute CLKIN_PERIOD 		of dcm_1	: label is "2.5" ;
attribute DLL_FREQUENCY_MODE 	of dcm_1	: label is "HIGH";
attribute CLKOUT_PHASE_SHIFT 	of dcm_1	: label is "FIXED";
attribute PHASE_SHIFT 		of dcm_1	: label is "64";

begin

Low <= '0' ;
locked <= locked0 and locked1 ;

dcm_0 : dcm 
--pragma translate_off
	generic map (
	DLL_FREQUENCY_MODE	=> cph,
	CLKOUT_PHASE_SHIFT 	=> cps,
	PHASE_SHIFT		=> ps0)
--pragma translate_on
	port map (
	clkin   => clkint,
	CLKFB   => clk,
	DSSEN 	=> low,
	PSINCDEC=> low,
	PSEN 	=> low,
	PSCLK 	=> low,
	RST     => rstin,
	CLK0    => clkdcm,
	CLK90   => open,
	CLK180  => open,
	CLK270  => open,
	CLK2X   => open,
	CLK2X180=> open,
	CLKDV   => open,
	CLKFX   => open,
	CLKFX180=> open,
	LOCKED  => locked0,
	PSDONE  => open,
	STATUS  => open);

dcm_1 : dcm 
--pragma translate_off
	generic map (
	DLL_FREQUENCY_MODE	=> cph,
	CLKOUT_PHASE_SHIFT 	=> cps,
	PHASE_SHIFT		=> ps1)
--pragma translate_on
	port map (
	clkin   => clkint,
	CLKFB   => clk90,
	DSSEN 	=> low,
	PSINCDEC=> low,
	PSEN 	=> low,
	PSCLK 	=> low,
	RST     => rstin,
	CLK0    => clk90dcm,
	CLK90   => open,
	CLK180  => open,
	CLK270  => open,
	CLK2X   => open,
	CLK2X180=> open,
	CLKDV   => open,
	CLKFX   => open,
	CLKFX180=> open,
	LOCKED  => locked1,
	PSDONE  => open,
	STATUS  => open);
	
ibufg1 : ibufg port map(i => clkin, o => clkint );

bufg1  : bufg  port map(i => clkdcm, o => clk );

bufg2  : bufg  port map(i => clk90dcm, o => clk90 );
	
drmodule : data_recovery_virtex2_fast port map(
	data 	=> datain,
	rst   	=> rstin,
	clk   	=> clk,
	clk90 	=> clk90,
	sdata 	=> dummyint,
	dv 	=> dvalidint );
		
process (clk, rstin)
begin
if rstin = '1' then
	dummy <= "00" ;
	dvalid <= "00" ;
elsif clk'event and clk = '1' then
	dummy <= dummyint ;
	dvalid <= dvalidint ;
end if ;
end process ;
end arch_top_v2_fast;


