------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 224 - Data Recovery
--
------------------------------------------------------------------------------
--
--  File name :       tb_top_v2_fast.vhd
--
--  Description :     Test bench for top_v2_fast.vhd
--
--  Date - revision : May 16th 2003 - v 2.2
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library ieee;
USE ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all ;

ENTITY tb_top_v2_fast IS END tb_top_v2_fast;

ARCHITECTURE rtl OF tb_top_v2_fast IS
component top_v2_fast
  PORT (
	clkin 	: in std_logic;
	rstin 	: in std_logic;
	datain 	: in std_logic;
	dummy 	: out std_logic_vector(1 downto 0);
	dvalid 	: out std_logic_vector(1 downto 0);
	locked 	: out std_logic); 
END component;

signal dataclk, clk, fastclk, match : std_logic := '0';
signal datainb, reset : std_logic := '1' ;
SIGNAL datain : std_logic_vector(63 downto 0) ;
signal dummy, dvalid : std_logic_vector(1 downto 0) ;
signal locked : std_logic ;
SIGNAL rxdata : std_logic_vector(63 downto 0) ;
SIGNAL var : std_logic_vector(1 downto 0) := "00"  ;
SIGNAL count, counth : integer := 0  ;

BEGIN

datainb <= datain(63) after 0.10 nS when var = "00" else	-- effectively adds large amounts of jitter
           datain(63) after 0.20 nS when var = "01" else
           datain(63) after 0.32 nS when var = "10" else
           datain(63)  ; 
	   
PROCESS (dataclk, locked, reset)
begin
if reset = '1' then
	datain <= "1111111011011100101110101001100001110110010101000011001000010000" ;
elsif dataclk'event and dataclk = '1' then
	datain <= datain(62 downto 0) & datain(63) ;
	var <= var + 1 ;
end if ;
end process ;

PROCESS (clk, locked, reset)
begin
if locked = '0' or reset = '1' then
	rxdata <= (others => '0' ) ;
elsif clk'event and clk = '1' then
	if dvalid = "01" then 
		count <= count + 1 ;
		rxdata <= rxdata(62 downto 0) & dummy(0) ;
	elsif dvalid = "11" then
		count <= count + 2 ;
		rxdata <= rxdata(61 downto 0) & dummy(0) & dummy(1) ;
	end if ;
	if rxdata = "1111111011011100101110101001100001110110010101000011001000010000" then
		match <= '1' ;
		counth <= count ;
		count <= 0 ;		
	else 
		match <= '0' ;
	end if ;
end if ;
end process ;

dataclk <= not dataclk after 1.250 ns ;	-- 400 MHz clock for data transmission
--clk <= not clk after 1.252 ns ;		-- slower clock for data reception
clk <= not clk after 1.248 ns ;		-- faster clock for data reception

reset <= '0' after 50 nS;

chip : top_v2_fast port map (
	clkin 	=> clk,
	rstin 	=> reset,
	datain 	=> datainb,
	dummy 	=> dummy,
	dvalid 	=> dvalid,
	locked	=> locked); 

END rtl;
