// Disclaimer : These designs are provided "as is" with
//              no warranty whatsoever, and xilinx specifically
//              disclaims any implied warranties of merchantability,
//              fitness for a particular purpose, or against infringement.
//
// Generic top level file to use data recovery module in a VirtexE
// uses one dll to generate two 90 degree phased clocks at up to 160 Mhz in a -7 .
//
`timescale 1 ps / 1ps

module top_ve (
	clkin,				// clock input
	rstin,				// reset input
	datain,			// data input
	dummy,				// dummy output for recovered data
	dvalid,			// recovered data valid when high
	locked				// DLL locked
	) ;

input clkin, rstin, datain ;
	
output [1:0] dummy, dvalid ;
output locked ;
reg [1:0] dummy, dvalid;

wire clk ;
wire clkdllo	;
wire clkint;
wire clk90;
wire clk90dllo;
wire [1:0] dummyint ;
wire [1:0] dvalidint ;

CLKDLL dll1(
	.CLKIN(clkint),
	.CLKFB(clk),
	.RST(rstin),
	.CLK0(clkdllo),
	.CLK90(clk90dllo),
	.LOCKED(locked));
	
IBUFG ibufg1(.I(clkin), .O(clkint) );

BUFG bufg1(.I(clkdllo), .O(clk) );
BUFG bufg2(.I(clk90dllo), .O(clk90) );

data_recovery_virtexe drmodule (
	.data(datain),
	.rst(rstin),
	.clk(clk),
	.clk90(clk90),
	.sdata(dummyint),
	.dv(dvalidint) );
		
always @(posedge clk or posedge rstin)
begin
if (rstin)
begin
	dummy = 2'b00;
	dvalid = 2'b00;
end
else
begin
	dummy = dummyint;
	dvalid = dvalidint;
end
end
endmodule


