-- Disclaimer : These designs are provided "as is" with
--              no warranty whatsoever, and xilinx specifically
--              disclaims any implied warranties of merchantability,
--              fitness for a particular purpose, or against infringement.
--
-- Generic top level file for VirtexE to use data recovery module
--
library IEEE;
use IEEE.std_logic_1164.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_ve is port (
	clkin 	: in std_logic;			-- clock input
	rstin 	: in std_logic;			-- reset input
	datain 	: in std_logic;			-- data input
	dummy 	: out std_logic_vector(1 downto 0);	-- dummy output for recovered data
	dvalid 	: out std_logic_vector(1 downto 0);	-- recovered data valid when high
	locked	: out std_logic); 		-- DLL locked
end top_ve;

architecture arch_top_ve of top_ve is
-- Low frequency DLL primitive
component clkdll port (
	clkin 	: in std_logic;
	clkFB 	: in std_logic;
	rst 	: in std_logic;
	clk0 	: out std_logic;
	clk90 	: out std_logic;
	clk180 	: out std_logic;
	clk270 	: out std_logic;
	clk2X 	: out std_logic;
	clkDV 	: out std_logic;
	LOCKED 	: out std_logic ); 
end component;
-- BUFG primitive
component bufg port (i : in std_logic; o : out std_logic); end component;
-- IBUFG primitive
component ibufg port (i : in std_logic; o : out std_logic); end component;
-- Data recovery module
component data_recovery_virtexe port (
	clk 	: in std_logic;
	clk90 	: in std_logic;
	data 	: in std_logic;
	rst 	: in std_logic;
	sdata 	: out std_logic_vector(1 downto 0) ;
	dv 	: out std_logic_vector(1 downto 0)) ; 
end component;

signal clk   	: std_logic;
signal clkdllo	: std_logic;
signal clkint	: std_logic;
signal clk90	: std_logic;
signal clk90dllo : std_logic;
signal dummyint : std_logic_vector(1 downto 0) ;
signal dvalidint: std_logic_vector(1 downto 0) ;

begin

dll1 : clkdll port map(
	clkin 	=> clkint,
	clkfb 	=> clk,
	rst   	=> rstin,
	clk0  	=> clkdllo,
	clk90  	=> clk90dllo,
	clk180 	=> open,
	clk270 	=> open,
	clk2X 	=> open,
	clkDV 	=> open,
	locked	=> locked);
	
ibufg1 : ibufg port map(i => clkin, o => clkint );

bufg1  : bufg  port map(i => clkdllo, o => clk );
bufg2  : bufg  port map(i => clk90dllo, o => clk90 );

drmodule : data_recovery_virtexe port map(
	data 	=> datain,
	rst   	=> rstin,
	clk   	=> clk,
	clk90 	=> clk90,
	sdata 	=> dummyint,
	dv 	=> dvalidint );
		
process (clk, rstin)
begin
if rstin = '1' then
	dummy <= "00" ;
	dvalid <= "00" ;
elsif clk'event and clk = '1' then
	dummy <= dummyint ;
	dvalid <= dvalidint ;
end if ;
end process ;
end arch_top_ve;


