------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 224 - Data Recovery
--
------------------------------------------------------------------------------
--
--  File name :       pn23.vhd
--
--  Description :     Generates 8 bits of a pn23 sequence every clock
--
--  Date - revision : May 16th 2003 - v 2.2
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity pn23 is Port ( 
	clk 	: in std_logic;				-- clock
        rst 	: in std_logic;				-- reset
	ce 	: in std_logic ;			-- clock enable
	init 	: in std_logic ;			-- load initial value
	prbs 	: out std_logic_vector(7 downto 0));	-- output
end pn23;

architecture arch_pn23 of pn23 is

signal q: std_logic_vector (22 downto 0);		-- internal 23 bit prbs

begin
prbs(7 downto 0) <= q(7 downto 0);			-- take low 8 bits for output

process (clk, rst)
begin
if rst = '1' then
	q <= "11111111111111111111111";
elsif clk'event and clk = '1' then
	if ce = '1' then
   		if init = '1' then 
   			q <= "10000011111000001000001" ; -- 41f041
   		else 
			q(0)  <= q(10) XOR q(15);
			q(1)  <= q(11) XOR q(16);
			q(2)  <= q(12) XOR q(17);
			q(3)  <= q(13) XOR q(18);
			q(4)  <= q(14) XOR q(19);
			q(5)  <= q(15) XOR q(20);
			q(6)  <= q(16) XOR q(21);
			q(7)  <= q(17) XOR q(22);
			q(8)  <= q(0);
			q(9)  <= q(1);
			q(10) <= q(2);
			q(11) <= q(3);
			q(12) <= q(4);
			q(13) <= q(5);
			q(14) <= q(6);
			q(15) <= q(7);
			q(16) <= q(8);
			q(17) <= q(9);
			q(18) <= q(10);
			q(19) <= q(11);
			q(20) <= q(12);
			q(21) <= q(13);
			q(22) <= q(14);
		end if ;
	end if ;
   end if;
end process;

end arch_pn23;







