------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 224 - Data Recovery
--
------------------------------------------------------------------------------
--
--  File name :       top_fast.vhd
--
--  Description :     Top level file to use fast data recovery module for LVDS demo board
-- 		      Doubles 210 clock for 420 Mbps transmission
--        	      Uses 2 x 210 and 2 x 210 + 90 degrees for data recovery
-- 		      data transmitted is a 23 bit prbs, match high when checked OK
--
--  Date - revision : May 16th 2003 - v 2.2
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_fast is port (
	rxclkin 		: in std_logic;				-- rx clock input
	txclkin			: in std_logic;				-- tx clock input
	rstin 			: in std_logic;				-- reset input
	switch 			: in std_logic;				-- input to force errors
	datain_p, datain_n	: in std_logic;				-- serial data input
	dataout_p, dataout_n	: out std_logic;			-- data output
	ui_tp			: out std_logic_vector(4 downto 0);	-- test points
	match 			: out std_logic); 			-- match output
end top_fast;

architecture arch_top_fast of top_fast is
-- DCM primitive
component dcm 
--pragma translate_off
	generic (
	DLL_FREQUENCY_MODE	: string ;
	CLKOUT_PHASE_SHIFT 	: string ;
	PHASE_SHIFT	 	: integer );
--pragma translate_on
	port (
	clkin   : in  std_logic ;
	CLKFB   : in  std_logic ;
	DSSEN 	: in  std_logic ;
	PSINCDEC: in  std_logic ;
	PSEN 	: in  std_logic ;
	PSCLK 	: in  std_logic ;
	RST     : in  std_logic ;
	CLK0    : out std_logic ;
	CLK90   : out std_logic ;
	CLK180  : out std_logic ;
	CLK270  : out std_logic ;
	CLK2X   : out std_logic ;
	CLK2X180: out std_logic ;
	CLKDV   : out std_logic ;
	CLKFX   : out std_logic ;
	CLKFX180: out std_logic ;
	LOCKED  : out std_logic ;
	PSDONE  : out std_logic ;
	STATUS  : out std_logic_vector(7 downto 0)); 
end component;
-- lvds output buffer primitive
component obufds_lvds_33 port(i : in std_logic; o, oB : out std_logic); end component;
-- lvds input buffer primitive
component ibufds_lvds_33 port(i, iB : in std_logic; o : out std_logic); end component;
-- CMOS33 input buffer primitive
component ibuf_lvcmos33 port (i : in std_logic; o : out std_logic); end component;
-- CMOS33 output buffer primitive
component obuf_lvcmos33 port (i : in std_logic; o : out std_logic); end component;
-- CMOS33 clock input buffer primitive
component ibufg_lvcmos33 port(i : in std_logic; o : out std_logic); end component;
-- BUFG primitive
component bufg port (i : in std_logic; o : out std_logic); end component;
-- IBUFG primitive
component ibufg port (i : in std_logic; o : out std_logic); end component;
-- Data recovery module
component data_recovery_virtex2_fast port (
	clk 		: in std_logic;
	clk90 		: in std_logic;
	data 		: in std_logic;
	rst 		: in std_logic;
	sdata 		: out std_logic_vector(1 downto 0) ;
	dv 		: out std_logic_vector(1 downto 0)) ; 
end component;
-- 23 bit prbs generator
component pn23 port ( 
	clk 		: in std_logic;				-- clock
        rst 		: in std_logic;				-- reset
	ce 		: in std_logic ;			-- clock enable
	init 		: in std_logic ;			-- load initial value
	prbs 		: out std_logic_vector(7 downto 0));	-- output
end component;

signal clk   		: std_logic;
signal rst   		: std_logic;
signal rstint  		: std_logic ;
signal low   		: std_logic;
signal high   		: std_logic;
signal rxclkdcm		: std_logic;
signal rxclkint		: std_logic;
signal txclkint		: std_logic;
signal rxclk90		: std_logic;
signal rxclk		: std_logic;
signal rxclk90dcm 	: std_logic;
signal matchint		: std_logic;
signal txclk		: std_logic;
signal rxdata 		: std_logic_vector(1 downto 0) ;
signal dvalid		: std_logic_vector(1 downto 0) ;
signal rxdcm0locked  	: std_logic ;
signal rxdcm1locked  	: std_logic ;
signal txdcmlocked  	: std_logic ;
signal data  		: std_logic ;
signal outdata  	: std_logic ;
signal dataint		: std_logic;
signal datain		: std_logic;
signal reg 		: std_logic_vector(15 downto 0) ;
signal txcount 		: std_logic_vector(2 downto 0) ;
signal rxreg 		: std_logic_vector(27 downto 0) ;
signal txreg 		: std_logic_vector(7 downto 0) ;
signal result 		: std_logic_vector(7 downto 0) ;
signal resultd 		: std_logic_vector(7 downto 0) ;
signal result2d 	: std_logic_vector(7 downto 0) ;
signal rxcount		: std_logic_vector(2 downto 0) ;
signal sync		: std_logic;
signal syncd		: std_logic;
signal mon		: std_logic_vector(4 downto 0) ;
signal tp		: std_logic_vector(4 downto 0) ;
signal txclkdcm		: std_logic ;
signal prbsgen		: std_logic_vector(7 downto 0) ;
signal prbschk		: std_logic_vector(7 downto 0) ;
signal pn0en		: std_logic;
signal pn0enpin		: std_logic;
signal pn1en		: std_logic;
signal framed		: std_logic;
signal passzero		: std_logic_vector(1 downto 0);
signal fr		: std_logic_vector(2 downto 0) ;
signal switchint	: std_logic;
signal rstdcm		: std_logic;

constant cps  		: string := "FIXED" ;
constant cph  		: string := "LOW" ;
constant ps0  		: integer := 0 ;
constant ps1  		: integer := 32 ;

attribute DLL_FREQUENCY_MODE 	: string;
attribute CLKOUT_PHASE_SHIFT 	: string;
attribute PHASE_SHIFT 		: string;
attribute CLKIN_PERIOD 		: string ;

attribute CLKIN_PERIOD 		of rx_dcm_0: label is "2.5" ;
attribute DLL_FREQUENCY_MODE 	of rx_dcm_0: label is "LOW";
attribute CLKOUT_PHASE_SHIFT 	of rx_dcm_0: label is "FIXED";
attribute PHASE_SHIFT 		of rx_dcm_0: label is "0";

attribute CLKIN_PERIOD 		of rx_dcm_1: label is "2.5" ;
attribute DLL_FREQUENCY_MODE 	of rx_dcm_1: label is "LOW";
attribute CLKOUT_PHASE_SHIFT 	of rx_dcm_1: label is "FIXED";
attribute PHASE_SHIFT 		of rx_dcm_1: label is "32";

attribute CLKIN_PERIOD 		of tx_dcm: label is "2.5" ;
attribute DLL_FREQUENCY_MODE 	of tx_dcm: label is "LOW";
attribute CLKOUT_PHASE_SHIFT 	of tx_dcm: label is "FIXED";
attribute PHASE_SHIFT 		of tx_dcm: label is "0";

begin

Low <= '0' ;
High <= '1' ;
rstdcm <= not rstint ;
rst <= not rstint or not rxdcm0locked or not rxdcm1locked or not txdcmlocked ;
tp <= mon ;

rx_dcm_0 : dcm 
--pragma translate_off
	generic map (
	DLL_FREQUENCY_MODE	=> cph,
	CLKOUT_PHASE_SHIFT 	=> cps,
	PHASE_SHIFT		=> ps0)
--pragma translate_on
	port map (
	clkin   => rxclkint,
	CLKFB   => rxclk,
	DSSEN 	=> low,
	PSINCDEC=> low,
	PSEN 	=> low,
	PSCLK 	=> low,
	RST     => rstdcm,
	CLK0    => open,
	CLK90   => open,
	CLK180  => open,
	CLK270  => open,
	CLK2X   => rxclkdcm,
	CLK2X180=> open,
	CLKDV   => open,
	CLKFX   => open,
	CLKFX180=> open,
	LOCKED  => rxdcm0locked,
	PSDONE  => open,
	STATUS  => open);

rx_dcm_1 : dcm 
--pragma translate_off
	generic map (
	DLL_FREQUENCY_MODE	=> cph,
	CLKOUT_PHASE_SHIFT 	=> cps,
	PHASE_SHIFT		=> ps1)
--pragma translate_on
	port map (
	clkin   => rxclkint,
	CLKFB   => rxclk90,
	DSSEN 	=> low,
	PSINCDEC=> low,
	PSEN 	=> low,
	PSCLK 	=> low,
	RST     => rstdcm,
	CLK0    => open,
	CLK90   => open,
	CLK180  => open,
	CLK270  => open,
	CLK2X   => rxclk90dcm,
	CLK2X180=> open,
	CLKDV   => open,
	CLKFX   => open,
	CLKFX180=> open,
	LOCKED  => rxdcm1locked,
	PSDONE  => open,
	STATUS  => open);

tx_dcm : dcm 
--pragma translate_off
	generic map (
	DLL_FREQUENCY_MODE	=> cph,
	CLKOUT_PHASE_SHIFT 	=> cps,
	PHASE_SHIFT		=> ps0)
--pragma translate_on
	port map (
	clkin   => txclkint,
	CLKFB   => txclk,
	DSSEN 	=> low,
	PSINCDEC=> low,
	PSEN 	=> low,
	PSCLK 	=> low,
	RST     => rstdcm,
	CLK0    => open,
	CLK90   => open,
	CLK180  => open,
	CLK270  => open,
	CLK2X   => txclkdcm,
	CLK2X180=> open,
	CLKDV   => open,
	CLKFX   => open,
	CLKFX180=> open,
	LOCKED  => txdcmlocked,
	PSDONE  => open,
	STATUS  => open);
		
ibufg0 : ibufg port map(i => rxclkin, o => rxclkint );
ibufg1 : ibufg port map(i => txclkin, o => txclkint );

tx_bufg    : bufg  port map(i => txclkdcm, o => txclk );
rx_bufg0   : bufg  port map(i => rxclkdcm, o => rxclk );
rx_bufg90  : bufg  port map(i => rxclk90dcm, o => rxclk90 );

lvds_data_ibuf : ibufds_lvds_33  port map (i => datain_p,iB => datain_n, o => data );	
lvds_data_obuf : obufds_lvds_33  port map (i => outdata,  o => dataout_p, oB => dataout_n );
	
rstinbuf 	: ibuf_lvcmos33	  port map(i => rstin, 		o => rstint );
swinbuf 	: ibuf_lvcmos33	  port map(i => switch, 	o => switchint );
matchobuf 	: obuf_lvcmos33	  port map(i => matchint,	o => match );
tp0_buf 	: obuf_lvcmos33	  port map(i => tp(0),		o => ui_tp(0) );
tp1_buf 	: obuf_lvcmos33	  port map(i => tp(1),		o => ui_tp(1) );
tp2_buf 	: obuf_lvcmos33	  port map(i => tp(2),		o => ui_tp(2) );
tp3_buf 	: obuf_lvcmos33	  port map(i => tp(3),		o => ui_tp(3) );
tp4_buf 	: obuf_lvcmos33	  port map(i => tp(4),		o => ui_tp(4) );


drmodule : data_recovery_virtex2_fast port map(
	data 	=> data,
	rst   	=> rst,
	clk   	=> rxclk,
	clk90 	=> rxclk90,
	sdata 	=> rxdata,
	dv 	=> dvalid);

pn0 : pn23 port map( 
	clk 	=> rxclk,
        rst 	=> rst,
	ce 	=> pn0enpin,
	init 	=> syncd,
	prbs 	=> prbschk) ;
	
process (rxclk, rst)
begin
if rst = '1' then
	rxreg <= (others => '0') ;
	result <= (others => '0') ;
	resultd <= (others => '0') ;
	result2d <= (others => '0') ;
	rxcount <= "111" ;
	matchint <= '0' ;
	sync <= '0' ;
	syncd <= '0' ;
	pn0en <= '0' ;
	pn0enpin <= '0' ;
	framed <= '0' ;
	passzero <= "00" ;
	fr <= "000" ;
elsif rxclk'event and rxclk = '1' then
	syncd <= sync ;
	pn0enpin <= pn0en or sync ;
	resultd <= result ;
	result2d <= resultd ;
	if rxreg(27 downto 20) = "11111100" then fr(2) <= '1'; else fr(2) <= '0'; end if ; -- look for 
	if rxreg(19 downto 12) = "11110000" then fr(1) <= '1'; else fr(1) <= '0'; end if ; -- FC F0 FF
	if rxreg(11 downto 4)  = "11111111" then fr(0) <= '1'; else fr(0) <= '0'; end if ;
	if dvalid = "01" then
		rxreg <= rxdata(0) & rxreg(27 downto 1) ;
	elsif dvalid = "11" then
		rxreg <= rxdata(1) & rxdata(0) & rxreg(27 downto 2) ;
	end if ;
	if framed = '1' and dvalid = "11" then
		rxcount <= rxcount + 2 ;
		if rxcount = "110" then
			passzero <= "01" ;
		elsif rxcount = "111" then
			passzero <= "10" ;
		else
			passzero <= "00" ;
		end if ;
	elsif framed = '1' and dvalid = "01" then 
		rxcount <= rxcount + 1 ;
		if rxcount = "111" then
			passzero <= "01" ;
		else
			passzero <= "00" ;
		end if ;
	else 
		passzero <= "00" ;
	end if ;
	if fr = "111" and framed = '0' then 
		sync <= '1' ;
		framed <= '1' ;
	else
		sync <= '0' ;
	end if ;
	if passzero = "01" then
		result <= rxreg(8 downto 1) ;
		pn0en <= '1' ;
	elsif passzero = "10" then
		result <= rxreg(7 downto 0) ;
		pn0en <= '1' ;
	else 
		pn0en <= '0' ;
	end if ;
	if result2d = prbschk then
		matchint <= '1' ;
	else 
		matchint <= '0' ;
	end if ;
end if ;
end process ;

pn1 : pn23 port map( 
	clk 	=> txclk,
        rst 	=> rst,
	ce 	=> pn1en,
	init 	=> low,
	prbs 	=> prbsgen) ;
	
process (txclk, rst)
begin
if rst = '1' then
	txreg <= (others => '0') ;
	txcount <= (others => '0') ;
	outdata <= '0' ;
	pn1en <= '0' ;
elsif txclk'event and txclk = '1' then
	txcount <= txcount + 1 ;
	if txcount = "111" then
		txreg <= prbsgen(7 downto 1) & (prbsgen(0) and switchint) ;
		pn1en <= '1' ;
	else
		txreg <= '0' & txreg(7 downto 1) ;
		pn1en <= '0' ;
	end if ;
	outdata <= txreg(0) ;
end if ;
end process ;
end arch_top_fast;


