------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 224 - Data Recovery
--
------------------------------------------------------------------------------
--
--  File name :       tb_top.vhd
--
--  Description :     Test bench for top level design
--
--  Date - revision : May 16th 2003 - v 2.2
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library ieee;
USE ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all ;

ENTITY tb_top IS END tb_top;

ARCHITECTURE rtl OF tb_top IS
component top
  PORT (
	rxclkin 		: in std_logic;				-- rx clock input
	txclkin			: in std_logic;				-- tx clock input
	switch 			: in std_logic;				-- input to force errors
	rstin 			: in std_logic;				-- reset input
	datain_p, datain_n	: in std_logic;				-- serial data input
	dataout_p, dataout_n	: out std_logic;			-- data output
	ui_tp			: out std_logic_vector(4 downto 0);	-- test points
	match 			: out std_logic); 			-- match output
END component;

signal txclk, rxclk,  match : std_logic := '0';
signal datad, datadB, reset : std_logic := '0' ;
SIGNAL data, dataB : std_logic ;
signal dummy, dvalid : std_logic_vector(1 downto 0) ;
signal locked : std_logic ;
SIGNAL var : std_logic_vector(1 downto 0) := "00"  ;
signal high : std_logic := '1' ;

BEGIN

datad <= data after 0.20 nS when var = "00" else
         data after 0.40 nS when var = "01" else
         data after 0.20 nS when var = "10" else
         data  ; 
datadB <= not datad ;

PROCESS (txclk, locked, reset)
begin
if reset = '0' then
elsif txclk'event and txclk = '1' then
	var <= var + 1 ;
end if ;
end process ;

txclk <= not txclk after 2.500 ns ;
rxclk <= not rxclk after 2.498 ns ;

reset <= '1' after 50 nS;

chip : top port map (
	rxclkin 		=> rxclk,
	txclkin			=> txclk,
	switch 			=> high,
	rstin 			=> reset,
	datain_p		=> datad,
	datain_n		=> datadB,
	dataout_p		=> data,
	dataout_n		=> dataB,
	ui_tp			=> open,
	match 			=> match);

END rtl;
